#include "m6955.h"
#include "definitions.h"


/* I2C Driver Handle */
DRV_HANDLE drvI2CHandle;

/* Ready transfer handle */
DRV_I2C_TRANSFER_HANDLE transferHandle;

/* Transfer status */
volatile APP_TRANSFER_STATUS transferStatus;

//DRV_I2C_BUFFER_EVENT i2cOpStatus;

/* I2C Driver TX buffer  */
uint8_t TXbuffer[10];
M6955_INIT_STATES InitState;
M6955_FREQ_STATES FreqState;

void I2CEventHandler(
        DRV_I2C_TRANSFER_EVENT event,
        DRV_I2C_TRANSFER_HANDLE transferHandle,
        uintptr_t context
        ) {
    APP_TRANSFER_STATUS* transferStatus = (APP_TRANSFER_STATUS*) context;

    if (event == DRV_I2C_TRANSFER_EVENT_COMPLETE) {
        if (transferStatus) {
            *transferStatus = APP_TRANSFER_STATUS_SUCCESS;
        }
    } else {
        if (transferStatus) {
            *transferStatus = APP_TRANSFER_STATUS_ERROR;
        }
    }
}

bool M6955_Init(void) {

    switch (InitState) {
        case INIT_OPEN:
        {
            CORETIMER_DelayMs(500);

            /* Open the I2C Driver */
            drvI2CHandle = DRV_I2C_Open(DRV_I2C_INDEX_0, DRV_IO_INTENT_READWRITE);

            /* Register the I2C Driver event Handler */
            DRV_I2C_TransferEventHandlerSet(
                    drvI2CHandle,
                    I2CEventHandler,
                    (uintptr_t) & transferStatus
                    );

            /* Write Transaction to M6955 */
            if (drvI2CHandle != (DRV_HANDLE) NULL) {
                InitState = INIT_REG00;
            } else {
                InitState = INIT_ERROR;
            }
            break;
        }

        case INIT_REG00:
        {
            transferStatus = APP_TRANSFER_STATUS_IN_PROGRESS;

            TXbuffer[0] = 0x00; // Select register
            TXbuffer[1] = 0b10000000; //b7=Power ON, b6=AM mode

            DRV_I2C_WriteTransferAdd(drvI2CHandle, M6955_ADDRESS,
                    (void *) &TXbuffer, 2,
                    &transferHandle
                    );

            InitState = INIT_REG06;

            break;
        }

        case INIT_REG06:
        {

            if (transferStatus == APP_TRANSFER_STATUS_SUCCESS) {
                transferStatus = APP_TRANSFER_STATUS_IN_PROGRESS;

                TXbuffer[0] = 0x06; // Select register
                TXbuffer[1] = 0b11000001; // b7:2=volume 48/64, b1=radio mode, b0=single speaker

                DRV_I2C_WriteTransferAdd(drvI2CHandle, M6955_ADDRESS,
                        (void *) &TXbuffer, 2,
                        &transferHandle
                        );

                InitState = INIT_REG07;
            }
            break;
        }

        case INIT_REG07:
        {

            if (transferStatus == APP_TRANSFER_STATUS_SUCCESS) {
                transferStatus = APP_TRANSFER_STATUS_IN_PROGRESS;

                TXbuffer[0] = 0x07; // Select register
                TXbuffer[1] = 0b00110001; // b5=base boost, b3:2=auto stereo

                DRV_I2C_WriteTransferAdd(drvI2CHandle, M6955_ADDRESS,
                        (void *) &TXbuffer, 2,
                        &transferHandle
                        );
                InitState = INIT_REG09;
            }

            break;
        }
        case INIT_REG09:
        {
            if (transferStatus == APP_TRANSFER_STATUS_SUCCESS) {
                transferStatus = APP_TRANSFER_STATUS_IN_PROGRESS;

                TXbuffer[0] = 0x09; // Select register
                TXbuffer[1] = 0b00000111; // b3=vol control resistor, b2=Xtal

                DRV_I2C_WriteTransferAdd(drvI2CHandle, M6955_ADDRESS,
                        (void *) &TXbuffer, 2,
                        &transferHandle
                        );
                InitState = INIT_COMPLETED;
            }

            break;
        }

        case INIT_COMPLETED:
        {
            if (transferStatus == APP_TRANSFER_STATUS_SUCCESS) {
                return true;
            }
            break;
        }

        default:
        {
            /* TODO: Handle error in application's state machine. */
            break;
        }
    }
    return false;
}

bool M6955_Freq(uint8_t band, uint16_t freq) {

    uint16_t ch;

    if (band == AM) ch = (freq / 9) * 3;
    else if (band == FM) ch = ((freq - 3000)*10) / 25;

    switch (FreqState) {
        case FREQ_REG03:
        {
            CORETIMER_DelayMs(300);

            if (transferStatus == APP_TRANSFER_STATUS_SUCCESS) {
                transferStatus = APP_TRANSFER_STATUS_IN_PROGRESS;

                TXbuffer[0] = 0x03; // Select register
                TXbuffer[1] = ch & 0xff; //b4:0=ch higher bits

                DRV_I2C_WriteTransferAdd(drvI2CHandle, M6955_ADDRESS,
                        (void *) &TXbuffer, 2,
                        &transferHandle
                        );

                FreqState = FREQ_REG02;
            }
            break;
        }

        case FREQ_REG02:
        {
            if (transferStatus == APP_TRANSFER_STATUS_SUCCESS) {
                transferStatus = APP_TRANSFER_STATUS_IN_PROGRESS;

                TXbuffer[0] = 0x02; // Select register
                TXbuffer[1] = (ch >> 8) | 0x40; // b7:0=ch lower bits

                DRV_I2C_WriteTransferAdd(drvI2CHandle, M6955_ADDRESS,
                        (void *) &TXbuffer, 2,
                        &transferHandle
                        );

                FreqState = FREQ_REG01_1;
            }
            break;
        }

        case FREQ_REG01_1:
        {
            if (transferStatus == APP_TRANSFER_STATUS_SUCCESS) {
                transferStatus = APP_TRANSFER_STATUS_IN_PROGRESS;

                TXbuffer[0] = 0x00; // Select register
                TXbuffer[1] = band; // b6=mode

                DRV_I2C_WriteTransferAdd(drvI2CHandle, M6955_ADDRESS,
                        (void *) &TXbuffer, 2,
                        &transferHandle
                        );

                FreqState = FREQ_REG01_2;
            }
            break;
        }

        case FREQ_REG01_2:
        {
            CORETIMER_DelayMs(1);
            if (transferStatus == APP_TRANSFER_STATUS_SUCCESS) {
                transferStatus = APP_TRANSFER_STATUS_IN_PROGRESS;

                TXbuffer[0] = 0x00; // Select register
                TXbuffer[1] = band | 0b00100000; // b6=mode, b5=Trigger tune process

                DRV_I2C_WriteTransferAdd(drvI2CHandle, M6955_ADDRESS,
                        (void *) &TXbuffer, 2,
                        &transferHandle
                        );

                FreqState = FREQ_REG01_3;
            }
            break;
        }

        case FREQ_REG01_3:
        {
            CORETIMER_DelayMs(1);
            if (transferStatus == APP_TRANSFER_STATUS_SUCCESS) {
                transferStatus = APP_TRANSFER_STATUS_IN_PROGRESS;

                TXbuffer[0] = 0x00; // Select register
                TXbuffer[1] = band; // b6= mode

                DRV_I2C_WriteTransferAdd(drvI2CHandle, M6955_ADDRESS,
                        (void *) &TXbuffer, 2,
                        &transferHandle
                        );

                FreqState = FREQ_COMPLETED;
            }
            break;
        }

        case FREQ_COMPLETED:
        {
            if (transferStatus == APP_TRANSFER_STATUS_SUCCESS) {
                return true;
            }
            break;
        }

        default:
        {
            /* TODO: Handle error in application's state machine. */
            break;
        }
    }
    return false;
}
/* *****************************************************************************
 End of File
 */
